/* Data for x86-64 version of processor capability information.
   Copyright (C) 2004 Free Software Foundation, Inc.
   This file is part of the GNU C Library.
   Contributed by Andreas Jaeger <aj@suse.de>, 2004.

   The GNU C Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   The GNU C Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the GNU C Library; if not, write to the Free
   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
   02111-1307 USA.  */

/* This information must be kept in sync with the _DL_HWCAP_COUNT and
   _DL_PLATFORM_COUNT definitions in procinfo.h.

   If anything should be added here check whether the size of each string
   is still ok with the given array size.

   All the #ifdefs in the definitions ar equite irritating but
   necessary if we want to avoid duplicating the information.  There
   are three different modes:

   - PROCINFO_DECL is defined.  This means we are only interested in
     declarations.

   - PROCINFO_DECL is not defined:

     + if SHARED is defined the file is included in an array
       initializer.  The .element = { ... } syntax is needed.

     + if SHARED is not defined a normal array initialization is
       needed.
  */

#ifndef PROCINFO_CLASS
#define PROCINFO_CLASS
#endif

  /* _dl_cache1size: size of L1 cache  */
#if !defined PROCINFO_DECL && defined SHARED
  ._dl_cache1size
#else
PROCINFO_CLASS long int _dl_cache1size
#endif
#ifndef PROCINFO_DECL
= 1024 * 64
#endif
#if !defined SHARED || defined PROCINFO_DECL
;
#else
,
#endif

  /* _dl_cache1sizehalf: 1/2 size of L1 cache  */
#if !defined PROCINFO_DECL && defined SHARED
  ._dl_cache1sizehalf
#else
PROCINFO_CLASS long int _dl_cache1sizehalf
#endif
#ifndef PROCINFO_DECL
= 1024 * 64 / 2
#endif
#if !defined SHARED || defined PROCINFO_DECL
;
#else
,
#endif

  /* _dl_cache2size: size of L2 cache  */
#if !defined PROCINFO_DECL && defined SHARED
  ._dl_cache2size
#else
PROCINFO_CLASS long int _dl_cache2size
#endif
#ifndef PROCINFO_DECL
= 1024 * 1024
#endif
#if !defined SHARED || defined PROCINFO_DECL
;
#else
,
#endif

  /* _dl_cache2size: 1/2 size of L2 cache  */
#if !defined PROCINFO_DECL && defined SHARED
  ._dl_cache2sizehalf
#else
PROCINFO_CLASS long int _dl_cache2sizehalf
#endif
#ifndef PROCINFO_DECL
= 1024 * 1024 / 2
#endif
#if !defined SHARED || defined PROCINFO_DECL
;
#else
,
#endif

#undef PROCINFO_DECL
#undef PROCINFO_CLASS
