/*
(C) 2002 Advanced Micro Devices, Inc. 
** YOUR USE OF THIS LIBRARY IS SUBJECT TO THE TERMS 
    AND CONDITIONS OF THE GNU LESSER GENERAL PUBLIC 
    LICENSE FOUND IN THE "README" FILE THAT IS INCLUDED WITH 
    THIS LIBRARY**
*/

#include "libm_amd.h"
#include "libm_util_amd.h"

#define USE_VAL_WITH_FLAGS
#define USE_SCALEDOUBLE_1
#define USE_SCALEUPDOUBLE1024
#include "libm_inlines_amd.h"
#undef USE_SCALEUPDOUBLE1024
#undef USE_SCALEDOUBLE_1
#undef USE_VAL_WITH_FLAGS

double __atan(double y)
{

  /* Some constants and split constants. */

  static double piby2 = 1.5707963267948966e+00; /* 0x3ff921fb54442d18 */
  double chi, clo, v, s, q, z;

  /* Find properties of argument y. */

  unsigned long uy, auy, yneg;
  GET_BITS_DP64(y, uy);
  auy = uy & ~SIGNBIT_DP64;
  yneg = (uy != auy);

  if (yneg) v = -y;
  else v = y;

  /* Argument reduction to range [-7/16,7/16] */

  if (auy > 0x4003800000000000) /* v > 39./16. */
    {

      if (auy > PINFBITPATT_DP64) return y + y; /* y is NaN */  
      else if (v > 0x4370000000000000)
	{ /* abs(y) > 2^56 => arctan(1/y) is 
	     insignificant compared to piby2 */
	  if (yneg) return val_with_flags(-piby2, AMD_F_INEXACT);
	  else return val_with_flags(piby2, AMD_F_INEXACT);
	}

      y = -1.0/v;
      /* (chi + clo) = arctan(infinity) */
      chi = 1.57079632679489655800e+00; /* 0x3ff921fb54442d18 */
      clo = 6.12323399573676480327e-17; /* 0x3c91a62633145c06 */
    }
  else if (auy > 0x3ff3000000000000) /* 39./16. > v > 19./16. */
    {
      y = (v-1.5)/(1.0+1.5*v);
      /* (chi + clo) = arctan(1.5) */
      chi = 9.82793723247329054082e-01; /* 0x3fef730bd281f69b */
      clo = 1.39033110312309953701e-17; /* 0x3c7007887af0cbbc */
    }
  else if (auy > 0x3fe6000000000000) /* 19./16. > v > 11./16. */
    {
      y = (v-1.)/(1.0+v);
      /* (chi + clo) = arctan(1.) */
      chi = 7.85398163397448278999e-01; /* 0x3fe921fb54442d18 */
      clo = 3.06161699786838240164e-17; /* 0x3c81a62633145c06 */
    }
  else if (auy > 0x3fdc000000000000) /* 11./16. > v > 7./16. */
    {
      y = (2*v-1.0)/(2.0+v);
      /* (chi + clo) = arctan(0.5) */
      chi = 4.63647609000806093515e-01; /* 0x3fddac670561bb4f */
      clo = 2.26987774529616809294e-17; /* 0x3c7a2b7f222f65e0 */
    }
  else  /* v < 7./16. */
    {
      y = v;
      chi = 0.0;
      clo = 0.0;
    }

  /* Core approximation: Remez(4,4) on [-7/16,7/16] */

  s = y*y;
  q = y*s*
       (0.268297920532545909e0 + 
	(0.447677206805497472e0 + 
	 (0.220638780716667420e0 + 
	  (0.304455919504853031e-1 + 
	    0.142316903342317766e-3*s)*s)*s)*s)/
       (0.804893761597637733e0 + 
	(0.182596787737507063e1 + 
	 (0.141254259931958921e1 + 
	  (0.424602594203847109e0 + 
	    0.389525873944742195e-1*s)*s)*s)*s);

  z = chi - ((q - clo) - y);

  if (yneg) z = -z;
  return z;
}

weak_alias (__atan, atan)
