/*
(C) 2002 Advanced Micro Devices, Inc. 
** YOUR USE OF THIS LIBRARY IS SUBJECT TO THE TERMS 
    AND CONDITIONS OF THE GNU LESSER GENERAL PUBLIC 
    LICENSE FOUND IN THE "README" FILE THAT IS INCLUDED WITH 
    THIS LIBRARY**
*/

#include "libm_amd.h"
#include "libm_util_amd.h"

float __floorf(float x)
{
  float r;
  int rexp, xneg;
  unsigned int ux, ax, ur, mask;

  GET_BITS_SP32(x, ux);
  ax = ux & (~SIGNBIT_SP32);
  xneg = (ux != ax);

  if (ax >= 0x4b800000)
    {
      /* abs(x) is either NaN, infinity, or >= 2^24 */
      if (ax > 0x7f800000)
        /* x is NaN */
        return x + x; /* Raise invalid if it is a signalling NaN */
      else
        return x;
    }
  else if (ax < 0x3f800000) /* abs(x) < 1.0 */
    {
      if (ax == 0x00000000)
        /* x is +zero or -zero; return the same zero */
        return x;
      else if (xneg) /* x < 0.0 */
        return -1.0F;
      else
        return 0.0F;
    }
  else
    {
      rexp = ((ux & EXPBITS_SP32) >> EXPSHIFTBITS_SP32) - EXPBIAS_SP32;
      /* Mask out the bits of r that we don't want */
      mask = (1 << (EXPSHIFTBITS_SP32 - rexp)) - 1;
      ur = (ux & ~mask);
      PUT_BITS_SP32(ur, r);
      if (xneg && (ux != ur))
        /* We threw some bits away and x was negative */
        return r - 1.0F;
      else
        return r;
    }
}

weak_alias (__floorf, floorf)
