/*
(C) 2002 Advanced Micro Devices, Inc. 
** YOUR USE OF THIS LIBRARY IS SUBJECT TO THE TERMS 
    AND CONDITIONS OF THE GNU LESSER GENERAL PUBLIC 
    LICENSE FOUND IN THE "README" FILE THAT IS INCLUDED WITH 
    THIS LIBRARY**
*/


#include "libm_amd.h"
#include "libm_util_amd.h"

#define USE_REMAINDER_PIBY2F_INLINE
#define USE_VAL_WITH_FLAGS
#define USE_NAN_WITH_FLAGS
#include "libm_inlines_amd.h"
#undef USE_VAL_WITH_FLAGS
#undef USE_NAN_WITH_FLAGS
#undef USE_REMAINDER_PIBY2F_INLINE

/* tan(x) approximation valid on the interval [-pi/4,pi/4]. 
   If recip is true return -1/tan(x) instead. */
static inline double tanf_piby4(double x, int recip)
{
  double r, t;

  /* Core Remez [1,2] approximation to tan(x) on the
     interval [0,pi/4]. */
  r = x*x;
  t = x + x*r*
    (0.385296071263995406715129e0 -
     0.172032480471481694693109e-1 * r) /
    (0.115588821434688393452299e+1 +
     (-0.51396505478854532132342e0 +
      0.1844239256901656082986661e-1 * r) * r);

  if (recip)
    return -1.0 / t;
  else
    return t;
}

float __tanf(float x)
{
  double r, dx;
  int region, xneg;

  unsigned long ux, ax;

  dx = x;

  GET_BITS_DP64(dx, ux);
  ax = (ux & ~SIGNBIT_DP64);

  if (ax <= 0x3fe921fb54442d18) /* abs(x) <= pi/4 */
    {
      if (ax < 0x3f80000000000000) /* abs(x) < 2.0^(-7) */
        {
          if (ax < 0x3f20000000000000) /* abs(x) < 2.0^(-13) */
            {
              if (ax == 0x0000000000000000)
                return dx;
              else
                return val_with_flags(dx, AMD_F_INEXACT);
            }
          else
            return dx + dx*dx*dx*0.333333333333333333;
        }
      else
        return tanf_piby4(dx, 0);
    }
  else if ((ux & EXPBITS_DP64) == EXPBITS_DP64)
    {
      /* x is either NaN or infinity */
      if (ux & MANTBITS_DP64)
        /* x is NaN */
        return dx + dx; /* Raise invalid if it is a signalling NaN */
      else
        /* x is infinity. Return a NaN */
        return nan_with_flags(AMD_F_INVALID);
    }

  xneg = (ux >> 63);

  if (xneg)
    x = -x;

  /* Reduce x into range [-pi/4,pi/4] */
  __remainder_piby2f_inline(x, ax, &r, &region);

  if (xneg)
    return -tanf_piby4(r, region & 1);
  else
    return tanf_piby4(r, region & 1);
}

weak_alias (__tanf, tanf)
